/*******************************************************************************
* File Name: SCBUART_SPI_UART.c
* Version 1.20
*
* Description:
*  This file provides the source code to the API for the SCB Component in
*  SPI and UART modes.
*
* Note:
*
*******************************************************************************
* Copyright 2013-2014, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions,
* disclaimers, and limitations in the end user license agreement accompanying
* the software package with which this file was provided.
*******************************************************************************/

#include "SCBUART_PVT.h"
#include "SCBUART_SPI_UART_PVT.h"

/***************************************
*        SPI/UART Private Vars
***************************************/

#if(SCBUART_INTERNAL_RX_SW_BUFFER_CONST)
    volatile uint32 SCBUART_rxBufferHead;
    volatile uint32 SCBUART_rxBufferTail;
    volatile uint8  SCBUART_rxBufferOverflow;
#endif /* (SCBUART_INTERNAL_RX_SW_BUFFER_CONST) */

#if(SCBUART_INTERNAL_TX_SW_BUFFER_CONST)
    volatile uint32 SCBUART_txBufferHead;
    volatile uint32 SCBUART_txBufferTail;
#endif /* (SCBUART_INTERNAL_TX_SW_BUFFER_CONST) */

#if(SCBUART_INTERNAL_RX_SW_BUFFER)
    /* Add one element to the buffer to receive full packet. One byte in receive buffer is always empty */
    volatile uint8 SCBUART_rxBufferInternal[SCBUART_RX_BUFFER_SIZE];
#endif /* (SCBUART_INTERNAL_RX_SW_BUFFER) */

#if(SCBUART_INTERNAL_TX_SW_BUFFER)
    volatile uint8 SCBUART_txBufferInternal[SCBUART_TX_BUFFER_SIZE];
#endif /* (SCBUART_INTERNAL_TX_SW_BUFFER) */


#if(SCBUART_RX_DIRECTION)

    /*******************************************************************************
    * Function Name: SCBUART_SpiUartReadRxData
    ********************************************************************************
    *
    * Summary:
    *  Retrieves the next data element from the receive buffer.
    *   - RX software buffer is disabled: Returns data element retrieved from
    *     RX FIFO. Undefined data will be returned if the RX FIFO is empty.
    *   - RX software buffer is enabled: Returns data element from the software
    *     receive buffer. Zero value is returned if the software receive buffer
    *     is empty.
    *
    * Parameters:
    *  None
    *
    * Return:
    *  Next data element from the receive buffer.
    *
    * Global Variables:
    *  Look into SCBUART_SpiInit for description.
    *
    *******************************************************************************/
    uint32 SCBUART_SpiUartReadRxData(void)
    {
        uint32 rxData = 0u;

        #if(SCBUART_INTERNAL_RX_SW_BUFFER_CONST)
            uint32 locTail;
        #endif /* (SCBUART_INTERNAL_RX_SW_BUFFER_CONST) */

        #if(SCBUART_CHECK_RX_SW_BUFFER)
        {
            if(SCBUART_rxBufferHead != SCBUART_rxBufferTail)
            {
                /* There is data in RX software buffer */

                /* Calculate index to read from */
                locTail = (SCBUART_rxBufferTail + 1u);

                if(SCBUART_RX_BUFFER_SIZE == locTail)
                {
                    locTail = 0u;
                }

                /* Get data from RX software buffer */
                rxData = SCBUART_GetWordFromRxBuffer(locTail);

                /* Change index in the buffer */
                SCBUART_rxBufferTail = locTail;
            }
        }
        #else
        {
            rxData = SCBUART_RX_FIFO_RD_REG; /* Read data from RX FIFO */
        }
        #endif

        return(rxData);
    }


    /*******************************************************************************
    * Function Name: SCBUART_SpiUartGetRxBufferSize
    ********************************************************************************
    *
    * Summary:
    *  Returns the number of received data elements in the receive buffer.
    *   - RX software buffer disabled: returns the number of used entries in
    *     RX FIFO.
    *   - RX software buffer enabled: returns the number of elements which were
    *     placed in the receive buffer.
    *
    * Parameters:
    *  None
    *
    * Return:
    *  Number of received data elements
    *
    *******************************************************************************/
    uint32 SCBUART_SpiUartGetRxBufferSize(void)
    {
        uint32 size;
        #if(SCBUART_INTERNAL_RX_SW_BUFFER_CONST)
            uint32 locHead;
        #endif /* (SCBUART_INTERNAL_RX_SW_BUFFER_CONST) */

        #if(SCBUART_CHECK_RX_SW_BUFFER)
        {
            locHead = SCBUART_rxBufferHead;

            if(locHead >= SCBUART_rxBufferTail)
            {
                size = (locHead - SCBUART_rxBufferTail);
            }
            else
            {
                size = (locHead + (SCBUART_RX_BUFFER_SIZE - SCBUART_rxBufferTail));
            }
        }
        #else
        {
            size = SCBUART_GET_RX_FIFO_ENTRIES;
        }
        #endif

        return(size);
    }


    /*******************************************************************************
    * Function Name: SCBUART_SpiUartClearRxBuffer
    ********************************************************************************
    *
    * Summary:
    *  Clears the receive buffer and RX FIFO.
    *
    * Parameters:
    *  None
    *
    * Return:
    *  None
    *
    *******************************************************************************/
    void SCBUART_SpiUartClearRxBuffer(void)
    {
        #if(SCBUART_INTERNAL_RX_SW_BUFFER_CONST)
            uint32 intSourceMask;
        #endif /* (SCBUART_INTERNAL_RX_SW_BUFFER_CONST) */

        #if(SCBUART_CHECK_RX_SW_BUFFER)
        {
            intSourceMask = SCBUART_SpiUartDisableIntRx();

            SCBUART_CLEAR_RX_FIFO;

            /* Flush RX software buffer */
            SCBUART_rxBufferHead     = SCBUART_rxBufferTail;
            SCBUART_rxBufferOverflow = 0u;

            /* End RX transfer */
            SCBUART_ClearRxInterruptSource(SCBUART_INTR_RX_ALL);

            SCBUART_SpiUartEnableIntRx(intSourceMask);
        }
        #else
        {
            SCBUART_CLEAR_RX_FIFO;
        }
        #endif
    }

#endif /* (SCBUART_RX_DIRECTION) */


#if(SCBUART_TX_DIRECTION)

    /*******************************************************************************
    * Function Name: SCBUART_SpiUartWriteTxData
    ********************************************************************************
    *
    * Summary:
    *  Places a data entry into the transmit buffer to be sent at the next available
    *  bus time.
    *  This function is blocking and waits until there is space available to put the
    *  requested data in the transmit buffer.
    *
    * Parameters:
    *  txDataByte: the data to be transmitted.
    *
    * Return:
    *  None
    *
    *******************************************************************************/
    void SCBUART_SpiUartWriteTxData(uint32 txData)
    {
        #if(SCBUART_INTERNAL_TX_SW_BUFFER_CONST)
            uint32 locHead;
            uint32 intSourceMask;
        #endif /* (SCBUART_INTERNAL_TX_SW_BUFFER_CONST) */

        #if(SCBUART_CHECK_TX_SW_BUFFER)
        {
            /* Head index to put data */
            locHead = (SCBUART_txBufferHead + 1u);

            /* Adjust TX software buffer index */
            if(SCBUART_TX_BUFFER_SIZE == locHead)
            {
                locHead = 0u;
            }

            while(locHead == SCBUART_txBufferTail)
            {
                /* Wait for space in TX software buffer */
            }

            /* TX software buffer has at least one room */

            if((SCBUART_txBufferHead == SCBUART_txBufferTail) &&
               (SCBUART_FIFO_SIZE != SCBUART_GET_TX_FIFO_ENTRIES))
            {
                /* TX software buffer is empty: put data directly in TX FIFO */
                SCBUART_TX_FIFO_WR_REG = txData;
            }
            /* Put data in TX software buffer */
            else
            {
                /* Clear old status of INTR_TX_NOT_FULL. It sets at the end of transfer when TX FIFO is empty. */
                SCBUART_ClearTxInterruptSource(SCBUART_INTR_TX_NOT_FULL);

                SCBUART_PutWordInTxBuffer(locHead, txData);

                SCBUART_txBufferHead = locHead;

                /* Enable interrupt to transmit */
                intSourceMask  = SCBUART_INTR_TX_NOT_FULL;
                intSourceMask |= SCBUART_GetTxInterruptMode();
                SCBUART_SpiUartEnableIntTx(intSourceMask);
            }
        }
        #else
        {
            while(SCBUART_FIFO_SIZE == SCBUART_GET_TX_FIFO_ENTRIES)
            {
                /* Block while TX FIFO is FULL */
            }

            SCBUART_TX_FIFO_WR_REG = txData;
        }
        #endif
    }


    /*******************************************************************************
    * Function Name: SCBUART_SpiUartPutArray
    ********************************************************************************
    *
    * Summary:
    *  Places an array of data into the transmit buffer to be sent.
    *  This function is blocking and waits until there is a space available to put
    *  all the requested data in the transmit buffer. The array size can be greater
    *  than transmit buffer size.
    *
    * Parameters:
    *  wrBuf:  pointer to an array with data to be placed in transmit buffer.
    *  count:  number of data elements to be placed in the transmit buffer.
    *
    * Return:
    *  None
    *
    *******************************************************************************/
    void SCBUART_SpiUartPutArray(const uint8 wrBuf[], uint32 count)
    {
        uint32 i;

        for(i=0u; i < count; i++)
        {
            SCBUART_SpiUartWriteTxData((uint32) wrBuf[i]);
        }
    }


    /*******************************************************************************
    * Function Name: SCBUART_SpiUartGetTxBufferSize
    ********************************************************************************
    *
    * Summary:
    * Returns the number of elements currently in the transmit buffer.
    *  - TX software buffer is disabled: returns the number of used entries in
    *    TX FIFO.
    *  - TX software buffer is enabled: returns the number of elements currently
    *    used in the transmit buffer. This number does not include used entries in
    *    the TX FIFO. The transmit buffer size is zero until the TX FIFO is
    *    not full.
    *
    * Parameters:
    *  None
    *
    * Return:
    *  Number of data elements ready to transmit.
    *
    *******************************************************************************/
    uint32 SCBUART_SpiUartGetTxBufferSize(void)
    {
        uint32 size;
        #if(SCBUART_INTERNAL_TX_SW_BUFFER_CONST)
            uint32 locTail;
        #endif /* (SCBUART_INTERNAL_TX_SW_BUFFER_CONST) */

        #if(SCBUART_CHECK_TX_SW_BUFFER)
        {
            /* Get current Tail index */
            locTail = SCBUART_txBufferTail;

            if(SCBUART_txBufferHead >= locTail)
            {
                size = (SCBUART_txBufferHead - locTail);
            }
            else
            {
                size = (SCBUART_txBufferHead + (SCBUART_TX_BUFFER_SIZE - locTail));
            }
        }
        #else
        {
            size = SCBUART_GET_TX_FIFO_ENTRIES;
        }
        #endif

        return(size);
    }


    /*******************************************************************************
    * Function Name: SCBUART_SpiUartClearTxBuffer
    ********************************************************************************
    *
    * Summary:
    *  Clears the transmit buffer and TX FIFO.
    *
    * Parameters:
    *  None
    *
    * Return:
    *  None
    *
    *******************************************************************************/
    void SCBUART_SpiUartClearTxBuffer(void)
    {
        #if(SCBUART_INTERNAL_TX_SW_BUFFER_CONST)
            uint32 intSourceMask;
        #endif /* (SCBUART_INTERNAL_TX_SW_BUFFER_CONST) */

        #if(SCBUART_CHECK_TX_SW_BUFFER)
        {
            intSourceMask = SCBUART_SpiUartDisableIntTx();

            SCBUART_CLEAR_TX_FIFO;

            /* Flush TX software buffer */
            SCBUART_txBufferHead = SCBUART_txBufferTail;

            /* End TX transfer if it is in progress */
            intSourceMask &= (uint32) ~SCBUART_INTR_TX_NOT_FULL;

            SCBUART_SpiUartEnableIntTx(intSourceMask);
        }
        #else
        {
            SCBUART_CLEAR_TX_FIFO;
        }
        #endif
    }

#endif /* (SCBUART_TX_DIRECTION) */


/*******************************************************************************
* Function Name: SCBUART_SpiUartDisableIntRx
********************************************************************************
*
* Summary:
*  Disables the RX interrupt sources.
*
* Parameters:
*  None
*
* Return:
*  Returns the RX interrupt sources enabled before the function call.
*
*******************************************************************************/
uint32 SCBUART_SpiUartDisableIntRx(void)
{
    uint32 intSource;

    intSource = SCBUART_GetRxInterruptMode();

    SCBUART_SetRxInterruptMode(SCBUART_NO_INTR_SOURCES);

    return(intSource);
}


/*******************************************************************************
* Function Name: SCBUART_SpiUartDisableIntTx
********************************************************************************
*
* Summary:
*  Disables TX interrupt sources.
*
* Parameters:
*  None
*
* Return:
*  Returns TX interrupt sources enabled before function call.
*
*******************************************************************************/
uint32 SCBUART_SpiUartDisableIntTx(void)
{
    uint32 intSourceMask;

    intSourceMask = SCBUART_GetTxInterruptMode();

    SCBUART_SetTxInterruptMode(SCBUART_NO_INTR_SOURCES);

    return(intSourceMask);
}


#if(SCBUART_SCB_MODE_UNCONFIG_CONST_CFG)
    /*******************************************************************************
    * Function Name: SCBUART_PutWordInRxBuffer
    ********************************************************************************
    *
    * Summary:
    *  Stores a byte/word into the RX buffer.
    *  Only available in the Unconfigured operation mode.
    *
    * Parameters:
    *  index:      index to store data byte/word in the RX buffer.
    *  rxDataByte: byte/word to store.
    *
    * Return:
    *  None
    *
    *******************************************************************************/
    void SCBUART_PutWordInRxBuffer(uint32 idx, uint32 rxDataByte)
    {
        /* Put data in buffer */
        if(SCBUART_ONE_BYTE_WIDTH == SCBUART_rxDataBits)
        {
            SCBUART_rxBuffer[idx] = ((uint8) rxDataByte);
        }
        else
        {
            SCBUART_rxBuffer[(uint32)(idx << 1u)]      = LO8(LO16(rxDataByte));
            SCBUART_rxBuffer[(uint32)(idx << 1u) + 1u] = HI8(LO16(rxDataByte));
        }
    }


    /*******************************************************************************
    * Function Name: SCBUART_GetWordFromRxBuffer
    ********************************************************************************
    *
    * Summary:
    *  Reads byte/word from RX buffer.
    *  Only available in the Unconfigured operation mode.
    *
    * Parameters:
    *  None
    *
    * Return:
    *  Returns byte/word read from RX buffer.
    *
    *******************************************************************************/
    uint32 SCBUART_GetWordFromRxBuffer(uint32 idx)
    {
        uint32 value;

        if(SCBUART_ONE_BYTE_WIDTH == SCBUART_rxDataBits)
        {
            value = SCBUART_rxBuffer[idx];
        }
        else
        {
            value  = (uint32) SCBUART_rxBuffer[(uint32)(idx << 1u)];
            value |= (uint32) ((uint32)SCBUART_rxBuffer[(uint32)(idx << 1u) + 1u] << 8u);
        }

        return(value);
    }


    /*******************************************************************************
    * Function Name: SCBUART_PutWordInTxBuffer
    ********************************************************************************
    *
    * Summary:
    *  Stores byte/word into the TX buffer.
    *  Only available in the Unconfigured operation mode.
    *
    * Parameters:
    *  idx:        index to store data byte/word in the TX buffer.
    *  txDataByte: byte/word to store.
    *
    * Return:
    *  None
    *
    *******************************************************************************/
    void SCBUART_PutWordInTxBuffer(uint32 idx, uint32 txDataByte)
    {
        /* Put data in buffer */
        if(SCBUART_ONE_BYTE_WIDTH == SCBUART_txDataBits)
        {
            SCBUART_txBuffer[idx] = ((uint8) txDataByte);
        }
        else
        {
            SCBUART_txBuffer[(uint32)(idx << 1u)]      = LO8(LO16(txDataByte));
            SCBUART_txBuffer[(uint32)(idx << 1u) + 1u] = HI8(LO16(txDataByte));
        }
    }


    /*******************************************************************************
    * Function Name: SCBUART_GetWordFromTxBuffer
    ********************************************************************************
    *
    * Summary:
    *  Reads byte/word from the TX buffer.
    *  Only available in the Unconfigured operation mode.
    *
    * Parameters:
    *  idx: index to get data byte/word from the TX buffer.
    *
    * Return:
    *  Returns byte/word read from the TX buffer.
    *
    *******************************************************************************/
    uint32 SCBUART_GetWordFromTxBuffer(uint32 idx)
    {
        uint32 value;

        if(SCBUART_ONE_BYTE_WIDTH == SCBUART_txDataBits)
        {
            value = (uint32) SCBUART_txBuffer[idx];
        }
        else
        {
            value  = (uint32) SCBUART_txBuffer[(uint32)(idx << 1u)];
            value |= (uint32) ((uint32) SCBUART_txBuffer[(uint32)(idx << 1u) + 1u] << 8u);
        }

        return(value);
    }

#endif /* (SCBUART_SCB_MODE_UNCONFIG_CONST_CFG) */


/* [] END OF FILE */
