/*******************************************************************************
* File Name: myUART_INT.c
* Version 2.30
*
* Description:
*  This file provides all Interrupt Service functionality of the UART component
*
* Note:
*  Any unusual or non-standard behavior should be noted here. Other-
*  wise, this section should remain blank.
*
********************************************************************************
* Copyright 2008-2012, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions,
* disclaimers, and limitations in the end user license agreement accompanying
* the software package with which this file was provided.
*******************************************************************************/

#include "myUART.h"
#include "CyLib.h"


/***************************************
* Custom Declratations
***************************************/
/* `#START CUSTOM_DECLARATIONS` Place your declaration here */

/* `#END` */

#if( (myUART_RX_ENABLED || myUART_HD_ENABLED) && \
     (myUART_RXBUFFERSIZE > myUART_FIFO_LENGTH))


    /*******************************************************************************
    * Function Name: myUART_RXISR
    ********************************************************************************
    *
    * Summary:
    *  Interrupt Service Routine for RX portion of the UART
    *
    * Parameters:
    *  None.
    *
    * Return:
    *  None.
    *
    * Global Variables:
    *  myUART_rxBuffer - RAM buffer pointer for save received data.
    *  myUART_rxBufferWrite - cyclic index for write to rxBuffer,
    *     increments after each byte saved to buffer.
    *  myUART_rxBufferRead - cyclic index for read from rxBuffer,
    *     checked to detect overflow condition.
    *  myUART_rxBufferOverflow - software overflow flag. Set to one
    *     when myUART_rxBufferWrite index overtakes
    *     myUART_rxBufferRead index.
    *  myUART_rxBufferLoopDetect - additional variable to detect overflow.
    *     Set to one when myUART_rxBufferWrite is equal to
    *    myUART_rxBufferRead
    *  myUART_rxAddressMode - this variable contains the Address mode,
    *     selected in customizer or set by UART_SetRxAddressMode() API.
    *  myUART_rxAddressDetected - set to 1 when correct address received,
    *     and analysed to store following addressed data bytes to the buffer.
    *     When not correct address received, set to 0 to skip following data bytes.
    *
    *******************************************************************************/
    CY_ISR(myUART_RXISR)
    {
        uint8 readData;
        uint8 increment_pointer = 0u;
        #if(CY_PSOC3)
            uint8 int_en;
        #endif /* CY_PSOC3 */

        /* User code required at start of ISR */
        /* `#START myUART_RXISR_START` */

        /* `#END` */

        #if(CY_PSOC3)   /* Make sure nested interrupt is enabled */
            int_en = EA;
            CyGlobalIntEnable;
        #endif /* CY_PSOC3 */

        readData = myUART_RXSTATUS_REG;

        if((readData & (myUART_RX_STS_BREAK | myUART_RX_STS_PAR_ERROR |
                        myUART_RX_STS_STOP_ERROR | myUART_RX_STS_OVERRUN)) != 0u)
        {
            /* ERROR handling. */
            /* `#START myUART_RXISR_ERROR` */

            /* `#END` */
        }

        while((readData & myUART_RX_STS_FIFO_NOTEMPTY) != 0u)
        {

            #if (myUART_RXHW_ADDRESS_ENABLED)
                if(myUART_rxAddressMode == (uint8)myUART__B_UART__AM_SW_DETECT_TO_BUFFER)
                {
                    if((readData & myUART_RX_STS_MRKSPC) != 0u)
                    {
                        if ((readData & myUART_RX_STS_ADDR_MATCH) != 0u)
                        {
                            myUART_rxAddressDetected = 1u;
                        }
                        else
                        {
                            myUART_rxAddressDetected = 0u;
                        }
                    }

                    readData = myUART_RXDATA_REG;
                    if(myUART_rxAddressDetected != 0u)
                    {   /* store only addressed data */
                        myUART_rxBuffer[myUART_rxBufferWrite] = readData;
                        increment_pointer = 1u;
                    }
                }
                else /* without software addressing */
                {
                    myUART_rxBuffer[myUART_rxBufferWrite] = myUART_RXDATA_REG;
                    increment_pointer = 1u;
                }
            #else  /* without addressing */
                myUART_rxBuffer[myUART_rxBufferWrite] = myUART_RXDATA_REG;
                increment_pointer = 1u;
            #endif /* End SW_DETECT_TO_BUFFER */

            /* do not increment buffer pointer when skip not adderessed data */
            if( increment_pointer != 0u )
            {
                if(myUART_rxBufferLoopDetect != 0u)
                {   /* Set Software Buffer status Overflow */
                    myUART_rxBufferOverflow = 1u;
                }
                /* Set next pointer. */
                myUART_rxBufferWrite++;

                /* Check pointer for a loop condition */
                if(myUART_rxBufferWrite >= myUART_RXBUFFERSIZE)
                {
                    myUART_rxBufferWrite = 0u;
                }
                /* Detect pre-overload condition and set flag */
                if(myUART_rxBufferWrite == myUART_rxBufferRead)
                {
                    myUART_rxBufferLoopDetect = 1u;
                    /* When Hardware Flow Control selected */
                    #if(myUART_FLOW_CONTROL != 0u)
                    /* Disable RX interrupt mask, it will be enabled when user read data from the buffer using APIs */
                        myUART_RXSTATUS_MASK_REG  &= (uint8)~myUART_RX_STS_FIFO_NOTEMPTY;
                        CyIntClearPending(myUART_RX_VECT_NUM);
                        break; /* Break the reading of the FIFO loop, leave the data there for generating RTS signal */
                    #endif /* End myUART_FLOW_CONTROL != 0 */
                }
            }

            /* Check again if there is data. */
            readData = myUART_RXSTATUS_REG;
        }

        /* User code required at end of ISR (Optional) */
        /* `#START myUART_RXISR_END` */

        /* `#END` */

        #if(CY_PSOC3)
            EA = int_en;
        #endif /* CY_PSOC3 */

    }

#endif /* End myUART_RX_ENABLED && (myUART_RXBUFFERSIZE > myUART_FIFO_LENGTH) */


#if(myUART_TX_ENABLED && (myUART_TXBUFFERSIZE > myUART_FIFO_LENGTH))


    /*******************************************************************************
    * Function Name: myUART_TXISR
    ********************************************************************************
    *
    * Summary:
    * Interrupt Service Routine for the TX portion of the UART
    *
    * Parameters:
    *  None.
    *
    * Return:
    *  None.
    *
    * Global Variables:
    *  myUART_txBuffer - RAM buffer pointer for transmit data from.
    *  myUART_txBufferRead - cyclic index for read and transmit data
    *     from txBuffer, increments after each transmited byte.
    *  myUART_rxBufferWrite - cyclic index for write to txBuffer,
    *     checked to detect available for transmission bytes.
    *
    *******************************************************************************/
    CY_ISR(myUART_TXISR)
    {

        #if(CY_PSOC3)
            uint8 int_en;
        #endif /* CY_PSOC3 */

        /* User code required at start of ISR */
        /* `#START myUART_TXISR_START` */

        /* `#END` */

        #if(CY_PSOC3)   /* Make sure nested interrupt is enabled */
            int_en = EA;
            CyGlobalIntEnable;
        #endif /* CY_PSOC3 */

        while((myUART_txBufferRead != myUART_txBufferWrite) &&
             ((myUART_TXSTATUS_REG & myUART_TX_STS_FIFO_FULL) == 0u))
        {
            /* Check pointer. */
            if(myUART_txBufferRead >= myUART_TXBUFFERSIZE)
            {
                myUART_txBufferRead = 0u;
            }

            myUART_TXDATA_REG = myUART_txBuffer[myUART_txBufferRead];

            /* Set next pointer. */
            myUART_txBufferRead++;
        }

        /* User code required at end of ISR (Optional) */
        /* `#START myUART_TXISR_END` */

        /* `#END` */

        #if(CY_PSOC3)
            EA = int_en;
        #endif /* CY_PSOC3 */

    }

#endif /* End myUART_TX_ENABLED && (myUART_TXBUFFERSIZE > myUART_FIFO_LENGTH) */


/* [] END OF FILE */
